		-- Directory and File Operations --

include sort.e

constant M_SEEK  = 19,
	 M_WHERE = 20,
	 M_DIR   = 22,
	 M_CURRENT_DIR = 23,
	 M_ALLOW_BREAK = 42,
	 M_CHECK_BREAK = 43

type file_number(integer f)
    return f >= 0
end type

type file_position(integer p)
    return p >= -1
end type

type boolean(integer b)
    return b = 0 or b = 1
end type

global function seek(file_number fn, file_position pos)
-- seeks to a byte position in the file, 
-- or to end of file if pos is -1
    return machine_func(M_SEEK, {fn, pos})
end function

global function where(file_number fn)
-- returns the current byte position in the file
    return machine_func(M_WHERE, fn)
end function

global constant 
	D_NAME = 1,
	D_ATTRIBUTES = 2,
	D_SIZE = 3,

	D_YEAR = 4,
	D_MONTH = 5,
	D_DAY = 6,

	D_HOUR = 7,
	D_MINUTE = 8,
	D_SECOND = 9
global function dir(sequence name)
-- returns directory information, given the name
-- of a file or directory. Format returned is:
-- {
--  {"name1", attributes, size, year, month, day, hour, minute, second},
--  {"name2", ...                                                     },
-- }
    return machine_func(M_DIR, name)
end function

global function current_dir()
-- returns name of current working directory
    return machine_func(M_CURRENT_DIR, 0)
end function

global procedure allow_break(boolean b)
-- If b is TRUE then allow control-c/control-break to 
-- terminate the program. If b is FALSE then don't allow it.
-- Initially they *will* terminate the program, but only when it
-- tries to read input from the keyboard.
    machine_proc(M_ALLOW_BREAK, b)
end procedure

global function check_break()
-- returns the number of times that control-c or control-break
-- were pressed since the last time check_break() was called
    return machine_func(M_CHECK_BREAK, 0)
end function

-- Generalized recursive directory walker

global constant W_BAD_PATH = -1 -- error code

function default_dir(sequence path)
-- Default directory sorting function for walk_dir().
-- * sorts by name *
    object d
    
    d = dir(path)
    if atom(d) then
	return d
    else
	-- sort by name
	return sort(d)
    end if
end function

-- override the dir sorting function with your own routine id
constant DEFAULT = -2
global integer my_dir 
my_dir = DEFAULT  -- it's better not to use routine_id() here,
		  -- or else users will have to bind with clear routine names

global function walk_dir(sequence path_name, integer your_function, 
			 integer scan_subdirs)
-- Generalized Directory Walker
-- Walk through a directory and (optionally) its subdirectories,
-- "visiting" each file and subdirectory. Your function will be called
-- via its routine id. The visits will occur in alphabetical order.
-- Your function should accept the path name and dir() entry for
-- each file and subdirectory. It should return 0 to keep going,
-- or an error code (greater than 0) to quit, or it can return
-- any sequence or atom other than 0 as a useful diagnostic value.
    object d, abort_now
    
    -- get the full directory information
    if my_dir = DEFAULT then
	d = default_dir(path_name)
    else
	d = call_func(my_dir, {path_name})
    end if
    if atom(d) then
	return W_BAD_PATH
    end if
    
    -- trim any trailing blanks or '\' characters from the path
    while find(path_name[length(path_name)], " \\") do
	path_name = path_name[1..length(path_name)-1]
    end while
    
    for i = 1 to length(d) do
	if find('d', d[i][D_ATTRIBUTES]) then
	    -- a directory
	    if not find(d[i][D_NAME], {".", ".."}) then
		abort_now = call_func(your_function, {path_name, d[i]})
		if not equal(abort_now, 0) then
		    return abort_now
		end if
		if scan_subdirs then
		    abort_now = walk_dir(path_name & '\\' & d[i][D_NAME],
					 your_function, scan_subdirs)
		    
		    if not equal(abort_now, 0) and 
		       not equal(abort_now, W_BAD_PATH) then
			-- allow BAD PATH, user might delete a file or directory 
			return abort_now
		    end if
		end if
	    end if
	else
	    -- a file
	    abort_now = call_func(your_function, {path_name, d[i]})
	    if not equal(abort_now, 0) then
		return abort_now
	    end if
	end if
    end for
    return 0
end function
